from tkinter import *

# import filedialog module
from tkinter import filedialog
from PIL import ImageTk, Image
# custom functions
from processing import *


root = Tk()
# root.geometry("700x500")
root.eval('tk::PlaceWindow . center')
root.title("Digital inline holographic microscope")


def add_reconstruction(z, hologram):
    """ Adds reconstructed image  in distance z (um) from the sensor to the dictionary of reconstructed images. """
    propagated_complex, elapsed = propagate(np.sqrt(hologram), wavelength, z * 1e-6, dxy)  # z in meters
    amplitude = complex2amplitude(propagated_complex)
    images[z] = amplitude
    display_images[z] = ImageTk.PhotoImage((Image.fromarray(amplitude)).resize((400, 300)))

    return elapsed


def step(z_step):
    global image_label
    global z
    global z_info
    global avg_time
    global elapsed_time

    z = z + z_step

    if z not in images:
        z_info.grid_forget()
        z_info = Label(root, text="Reconstructing...(expected time = %.2f s)" % avg_time, bd=1, relief=SUNKEN, anchor=CENTER)
        z_info.grid(row=3, column=0, columnspan=6, sticky=W + E)
        root.update()
        elapsed_time = add_reconstruction(z, hologram)
        avg_time = (avg_time * (len(images) - 1) + elapsed_time) / len(images)

    image_label = Label(root, image=display_images[z])
    image_label.grid(row=0, column=0, columnspan=6)

    z_info.grid_forget()
    z_info = Label(text="z = " + str(z) + " um", bd=1, relief=SUNKEN, anchor=CENTER)
    z_info.grid(row=3, column=0, columnspan=6, sticky=W + E)


def initialize():
    global z
    global wavelength
    global dxy
    global holopath
    global elapsed_time
    global image_label
    global hologram
    global frame
    global button_zoom_in
    global zoom_label
    global button_zoom_out

    wavelength = int(wvln_input.get()) * 1e-9
    dxy = float(dxy_input.get()) * 1e-6
    z = int(z_input.get())

    for widget in frame.winfo_children():
        widget.grid_forget()

    # frame = LabelFrame(root, padx=50, pady=50)
    status_label = Label(frame, text="Loading image...")
    status_label.grid(row=0, column=0, pady=50, padx=20)
    frame.update()
    # initialize with taking hologram
    _, green1, _, _, _, _, rotated = process_image(holopath)
    hologram = green1

    status_label.forget()
    status_label = Label(frame, text="Reconstructing...")
    status_label.grid(row=0, column=0, pady=50, padx=20)
    frame.update()
    elapsed_time = add_reconstruction(z, hologram)
    avg_time = elapsed_time

    image_label = Label(root, image=display_images[z])

    frame.destroy()

    image_label.grid(row=0, column=0, columnspan=6)
    button_back100.grid(row=1, column=0)
    button_back10.grid(row=1, column=1)
    button_exit.grid(row=1, column=2, columnspan=1)
    button_forward10.grid(row=1, column=3, pady=5)
    button_forward100.grid(row=1, column=4, pady=5)
    button_zoom_out.grid(row=2, column=1)
    zoom_label.grid(row=2, column=2)
    button_zoom_in.grid(row=2, column=3)
    z_info.grid(row=3, column=0, columnspan=6, sticky=W + E)


def zoom_img(step):
    global zoom

    zoom = max(1, zoom + step)

    img = crop(images[z]) # based on global zoom and rel_coordinates

    display_images[z] = ImageTk.PhotoImage((Image.fromarray(img)).resize((400, 300)))

    image_label = Label(root, image=display_images[z])
    image_label.grid(row=0, column=0, columnspan=6)

    zoom_label = Label(root, text="Zoom: %d" % (zoom * 100) + "%")
    zoom_label.grid(row=2, column=2)

    return

def crop(img):
    (ydim, xdim) = img.shape
    xdiff = np.floor((xdim - xdim/zoom) / 2)
    ydiff = np.floor((ydim - ydim/zoom) / 2)

    lower = int(max(ydiff + rel_coordinates['y'], 0))
    upper = int(min(ydiff + rel_coordinates['y'] + ydim/zoom, ydim))
    left = int(max(xdiff + rel_coordinates['x'], 0))
    right = int(min(xdiff + rel_coordinates['x'] + xdim/zoom, xdim))

    img = img[lower:upper, left:right]
    return img


# Function for opening the file explorer window
def browseFiles():
    global holopath

    holopath = filedialog.askopenfilename(initialdir="/",
                                          title="Select a File",
                                          filetypes=(("Raw Images", "*.jpg"),
                                                     ("all files", "*.*")))

    # Change label contents
    label_file_explorer.configure(text="File Opened: " + holopath)


# ____________________ Closing functions _________________________
def exit_program():
    exit_window = Toplevel(root)
    exit_window.title("Exit DLHM")
    # exit_window.geometry("200x200")
    question_label = Label(exit_window, text="Do you want to save reconstructed images?", pady=10)
    button_save_all = Button(exit_window,
                         text="Save all",
                         command=lambda: save_all(exit_window),
                         pady=10)
    button_save_last = Button(exit_window,
                         text="Save current",
                         command=lambda: save_last(exit_window),
                         pady=10)
    button_discard = Button(exit_window,
                            text="Discard",
                            command=lambda: discard_all(exit_window),
                            pady=10)

    question_label.grid(row=0, column=0, columnspan=3)
    button_save_all.grid(row=1, column=0, pady=10)
    button_save_last.grid(row=1, column=1, pady=10)
    button_discard.grid(row=1, column=2, pady=10)
    exit_window.update()
    root.update()


def save_all(exit_window):
    path = os.path.dirname(holopath)
    for z in images:
        filename = "propagated_z_%.0f" % z + "_um.png"
        save_grayscale_image(images[z], path=path + "/reconstructed/", name=filename)
    exit_window.quit()
    root.quit()


def save_last(exit_window):
    path = os.path.dirname(holopath)
    filename = "propagated_z_%.0f" % z + "_um.png"
    save_grayscale_image(images[z], path=path + "/reconstructed/", name=filename)
    exit_window.quit()
    root.quit()


def discard_all(exit_window):
    exit_window.quit()
    root.quit()


# ___________________ Initialize DESKTOP ____________________________
images = {}  # dict with images, key = distance z from sensor in um
display_images = {}
rel_coordinates = dict([('x', 0), ('y', 0)])

frame = LabelFrame(root, text="Enter parameters:", padx=50, pady=50)
# variables to get from the user
wvln_label = Label(frame, text="Wavelength [nm]: ", )
wvln_input = Entry(frame, width=10)
wvln_input.insert(0, 532)  # nm

dxy_label = Label(frame, text="Pixel size [um]: ")
dxy_input = Entry(frame, width=10)  # imager pixel size (virtual pixel size)
dxy_input.insert(0, 1.12 * 2)  # um

z_label = Label(frame, text="Distance z from sensor [um]: ")
z_input = Entry(frame, width=10)  # base z to reconstruct first
z_input.insert(0, 5500)  # um

label_file_explorer = Label(frame,
                            text="No raw hologram selected",
                            width=100,
                            height=2)

button_explore = Button(frame,
                        text="Browse Files",
                        command= browseFiles)

take_image_btn = Button(frame, text="Load image and reconstruct", pady=5, command=initialize)


# Positioning elements in Initialize frame
frame.pack(padx=5, pady=5)
wvln_label.grid(row=0, column=0, sticky=E)
wvln_input.grid(row=0, column=1, sticky=W)
dxy_label.grid(row=1, column=0, sticky=E)
dxy_input.grid(row=1, column=1, sticky=W)
z_label.grid(row=2, column=0, sticky=E)
z_input.grid(row=2, column=1, sticky=W)
label_file_explorer.grid(row=3, column=0, columnspan=2)
button_explore.grid(row=4, column=0, columnspan=2)
take_image_btn.grid(row=5, column=0, columnspan=2, pady=10)


# _____________________ Reconstruct window setup _____________________________
# initialize global variables
z = 5500
dxy = 1.12 * 1e-6
# path = "/home/dihmsudo/Desktop/dihm/data/GUI_data/"
avg_time = 0
zoom = 1

button_back100 = Button(root, text="-100 um", command=lambda: step(-100))
button_back10 = Button(root, text="-10 um", command=lambda: step(-10))
button_exit = Button(root, text="Exit program", command=exit_program)
button_forward10 = Button(root, text="+10 um", command=lambda: step(10))
button_forward100 = Button(root, text="+100 um", command=lambda: step(100))

zoom_in_img = ImageTk.PhotoImage(Image.open("./images/zoom_in.png").resize((25, 25)))
zoom_out_img = ImageTk.PhotoImage(Image.open("./images/zoom_out.png").resize((25, 25)))
button_zoom_in = Button(root, image=zoom_in_img, command=lambda: zoom_img(0.5))
zoom_label = Label(root, text="Zoom: %d" % (zoom*100) + "%")
button_zoom_out = Button(root, image=zoom_out_img, command=lambda: zoom_img(-0.5))

image_label = Label()
z_info = Label(root, text="z = " + str(z) + " um", bd=1, relief=SUNKEN, anchor=CENTER)

root.mainloop()
