# image processing
from picamraw import PiRawBayer, PiCameraVersion
import os
import errno
from matplotlib import image as mplim

from reconstruction import *


def process_image(holopath):
    path = os.path.dirname(holopath)
    raw_bayer = PiRawBayer(
        filepath=holopath,
        camera_version=PiCameraVersion.V2,
    )
    bayer_array = raw_bayer.bayer_array

    # BGGR
    ((ry, rx), (gy, gx), (Gy, Gx), (by, bx)) = ((1, 1), (0, 1), (1, 0), (0, 0))
    red = bayer_array[ry::2, rx::2]  # Red
    green1 = bayer_array[gy::2, gx::2]  # Green 1
    green2 = bayer_array[Gy::2, Gx::2]  # Green 2
    blue = bayer_array[by::2, bx::2]  # Blue

    # All green pixels diagonally
    green_diag = np.zeros(bayer_array.shape, dtype=bayer_array.dtype)
    green_diag[gy::2, gx::2] = green1
    green_diag[Gy::2, Gx::2] = green2

    rows = green_diag.shape[0]
    cols = green_diag.shape[1]
    N = min(rows, cols)
    M = max(rows, cols)
    crop = (M - N) // 2
    green_diag_square = green_diag[:, crop:M - crop]

    # Green pixels rotated'
    rotated = np.zeros((N, N))
    for row in range(N):
        for col in range(N):
            if (row + col) % 2 == 1:  # odd indexes green
                new_row = (row + col) // 2
                new_col = (N - 1 - (row - col)) // 2
                rotated[new_row, new_col] = green_diag_square[row, col]

    save_grayscale_image(green1, path + "/processed/", name="holo_green1.png")

    return red, green1, green2, blue, green_diag, green_diag_square, rotated


def save_grayscale_image(array, path, name):
    if not os.path.exists(os.path.dirname(path)):
        try:
            os.makedirs(os.path.dirname(path))
        except OSError as exc:  # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise
    outfile = path + name
    print("saving " + outfile)
    mplim.imsave(outfile, array, cmap='gray')
