import numpy as np
from timeit import default_timer as timer


def propagate(input_img, wvlen, z, dxy):
    # angular spectrum propagation method based on Rayleigh-Sommerfeld diffraction integral without approximations
    tic = timer()
    # get image size, rows M, columns N,  must be even
    M, N = input_img.shape

    # grid of Fourier frequencies in directions x, resp. y at the sensor plane
    _x1 = np.arange(0, N / 2)
    _x2 = np.arange(N / 2, 0, -1)
    _y1 = np.arange(0, M / 2)
    _y2 = np.arange(M / 2, 0, -1)
    _x = np.concatenate([_x1, _x2])
    _y = np.concatenate([_y1, _y2])
    x, y = np.meshgrid(_x, _y)
    fx, fy = x / (dxy * N), y / (dxy * M)
    # Fourier frequency in z direction
    fz = (fx * fx) + (fy * fy)

    # FT of at z=0
    F_E0 = np.fft.fft2(np.fft.fftshift(input_img))

    # fourier image of the impulse response of free space propagation
    F_h = np.exp(-1j * np.pi * wvlen * z * fz)

    # propagated complex amplitude of electric field in distance z
    E = np.fft.ifftshift(np.fft.ifft2(F_E0 * F_h))

    elapsed_time = timer() - tic

    return E, elapsed_time


def complex2amplitude(propagated_complex):
    amp = np.abs(propagated_complex) ** 2
    amp = np.sqrt(amp)*10 # better contrast than original amplitude

    amp = np.clip(amp, 0, 255)  # prevent rollover when converting to 8 bit
    amplitude = amp.astype('uint8')

    return amplitude
